﻿using System.ComponentModel;
using System.Reflection;
using System.Text;

namespace Recogmission.ResharperPlugin.CodeCleanup
{
    /// <summary>
    /// Defines options for code cleanup module.
    /// </summary>
    public class BaseCodeCleanupsOptions
    {
        #region Constructors and Destructors

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether need to use CLR types.
        /// </summary>
        [DisplayName( "Use CLR type names" )]
        public bool UseClrTypeNames
        {
            get;
            set;
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Returns a concatenated summary of the current options settings.
        /// </summary>
        /// <returns>
        /// A String of the options.
        /// </returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            var properties = GetType().GetProperties();

            for( var i = 0; i < properties.Length; i++ )
            {
                var property = properties[i];
                if( i > 0 )
                {
                    sb.Append( ", " );
                }

                sb.Append( GetPropertyDecription( property ) );
            }

            return sb.ToString();
        }

        #endregion

        #region Methods

        /// <summary>
        /// Builds a string representation of the property value.
        /// </summary>
        /// <param name="propertyInfo">
        /// The <see cref="PropertyInfo"/> to build the description for.
        /// </param>
        /// <returns>
        /// The string representation.
        /// </returns>
        private string GetPropertyDecription( PropertyInfo propertyInfo )
        {
            var propertyValue = propertyInfo.GetValue( this, null ).ToString();

            var propName = string.Empty;
            var propValue = string.Empty;
            var displayNameAttributes =
                (DisplayNameAttribute[]) propertyInfo.GetCustomAttributes( typeof( DisplayNameAttribute ), false );
            if( displayNameAttributes.Length == 1 )
            {
                propName = displayNameAttributes[0].DisplayName;
            }

            if( propertyInfo.PropertyType == typeof( bool ) )
            {
                propValue = propertyValue == "True" ? "Yes" : "No";
            }
            else
            {
                var field = propertyInfo.PropertyType.GetField( propertyValue );

                if( field != null )
                {
                    var descriptionAttributes =
                        (DescriptionAttribute[]) field.GetCustomAttributes( typeof( DescriptionAttribute ), false );
                    if( descriptionAttributes.Length == 1 )
                    {
                        propValue = descriptionAttributes[0].Description;
                    }
                }
            }

            return string.Format( "{0} = {1}", propName, propValue );
        }

        #endregion
    }
}